@@ -47,7 +47,7 @@ if($G{helpme} || (!$G{usefile} && scalar @ARGV == 0 && !$G{ipfile}))
 	{pod2usage(-exitstatus => 0, -verbose => 2);}
 
 print STDERR "FastScan enabled\n" if($G{verbose} > 0 && $G{fast});
-print "\nportinfo.pl - ( http://npx.sourceforge.net )\n",
+print "\nportinfo.pl - ( http://www.nmapparser.com )\n",
 	('-'x50),"\n\n";
 
 
@@ -332,7 +332,7 @@ L<http://sourceforge.net/tracker/?group_id=97509&atid=618345>
 
 L<Nmap::Parser>
 
-The Nmap::Parser page can be found at: L<http://npx.sourceforge.net/>.
+The Nmap::Parser page can be found at: L<http://www.nmapparser.com>.
 It contains the latest developments on the module. The nmap security scanner
 homepage can be found at: L<http://www.insecure.org/nmap/>.
 
